import 'dart:developer';
import 'dart:io';
import 'dart:typed_data';
import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_storage/firebase_storage.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_firebase/nui_firebase.dart';
import 'package:http/http.dart' as http;

class NUIFireStorage implements NUIFireComponent{
  NUIFireStorage._builder(NUIFireStorageBuilder builder){
    _bucket = builder._bucket;
    _storage = builder._storage;
    _instance = this;
    print("NUIFireStorage instance built");
  }

  String _bucket;
  FirebaseStorage _storage;
  static NUIFireStorage _instance;

  static NUIFireStorage get(){
    if(_instance == null) throw Exception("NUIFireStorage -> You have not initialized NUIFireStorage with builder");
    return _instance;
  }

  FirebaseStorage getFirebaseStorage(){
    return _storage;
  }

  Future<FireResult<TaskSnapshot>> uploadToPath(File file, String path, {SettableMetadata metadata}) async{
    return upload(file, _storage.ref().child(path), metadata: metadata);
  }

  Future<FireResult<TaskSnapshot>> upload(File file, Reference storageReference, {SettableMetadata metadata}) async{
    TaskSnapshot snapshot;
    print("NUIFireStorage: Uploading file of path : ${file.path} to reference : ${storageReference.fullPath}");
    try {
      final uploadTask = storageReference.putFile(file, metadata);
      snapshot = await uploadTask.whenComplete(() => null);
      // snapshot = uploadTask.snapshot;
      print("NUIFireStorage: Uploaded file of path : ${file.path} to reference : ${storageReference.fullPath} successful");
    }catch(e){
      print("NUIFireStorage: Failed to upload file of path : ${file.path} with error : ${e.toString().substring(0, 200)}");
      snapshot = null;
    }

    if(snapshot != null){
      return FireResult(success: true, message: "File uploaded successfully", data: snapshot);
    }
    else{
      return FireResult(success: false, message: "Failed to upload file", data: snapshot);
    }
  }

  Future<FireResult<TaskSnapshot>> uploadBytes(Uint8List bytes, Reference storageReference, {SettableMetadata metadata}) async{
    TaskSnapshot snapshot;
    print("NUIFireStorage: Uploading file to reference : ${storageReference.fullPath}");
    try {
      final uploadTask = storageReference.putData(bytes, metadata);
      snapshot = await uploadTask.whenComplete(() => null);
      // snapshot = uploadTask.snapshot;
      print("NUIFireStorage: Uploaded file to reference : ${storageReference.fullPath} successful");
    }catch(e){
      print("NUIFireStorage: Failed to upload file with error : ${e.toString().substring(0, 200)}");
      snapshot = null;
    }

    if(snapshot != null){
      return FireResult(success: true, message: "File uploaded successfully", data: snapshot);
    }
    else{
      return FireResult(success: false, message: "Failed to upload file", data: snapshot);
    }
  }

  Future<FireResult<File>> download(Reference storageReference, String filename) async{
    File finalFile;
    try {
      final String url = await storageReference.getDownloadURL();
      final http.Response downloadData = await http.get(Uri.parse(url));
      final Directory systemTempDir = Directory.systemTemp;
      File tempFile = File('${systemTempDir.path}/$filename');
      if (tempFile.existsSync()) {
        await tempFile.delete();
      }
      await tempFile.create();
      assert(await tempFile.readAsString() == "");
      final DownloadTask task = storageReference.writeToFile(tempFile);
      final snapshot = await task.whenComplete(() => null);
      final int byteCount = (snapshot).totalBytes;
      final String tempFileContents = await tempFile.readAsString();

      final String fileContents = downloadData.body;
      final String name = storageReference.name;
      final String bucket = storageReference.bucket;
      final String path = storageReference.fullPath;

      finalFile = tempFile;
    }
    catch(e){
      log("NUIFireStorage -> Failed to download file to $filename");
      finalFile = null;
    }

    if(finalFile != null){
      return FireResult(success: true, message: "File downloaded successfully to ${finalFile.path}", data: finalFile);
    }
    else{
      return FireResult(success: false, message: "Failed to download file");
    }
  }

  @override
  String name() => "NUIFireStorage";
}

class NUIFireStorageBuilder implements NUIFireBuilder<NUIFireStorage>{
  String _bucket;
  String _androidAppId;
  String _iosAppId;
  FirebaseStorage _storage;

  NUIFireStorageBuilder bucket(String bucket){
    _bucket = bucket;
    return this;
  }

  NUIFireStorageBuilder androidAppId(String androidAppId){
    _androidAppId = androidAppId;
    return this;
  }

  NUIFireStorageBuilder iosAppId(String iosAppId){
    _iosAppId = iosAppId;
    return this;
  }

  @override
  NUIFireStorage build(FirebaseApp app){
    final FirebaseStorage storage = FirebaseStorage.instanceFor(app: app, bucket: _bucket);
    _storage = storage;
    return NUIFireStorage._builder(this);
  }
}